@@ -34,6 +34,8 @@
 #include "declarationid.h"
 #include "indexedducontext.h"
 
+#include "navigation/abstractnavigationwidget.h"
+
 class QWidget;
 
 namespace KDevelop
@@ -86,16 +88,16 @@ public:
    *
    * If the parent is in the symbol table and the context is not anonymous, it will also be added to the symbol table. You nead a write-lock to the DUChain then
    */
-  explicit DUContext(const RangeInRevision& range, DUContext* parent = 0, bool anonymous = false);
+  explicit DUContext(const RangeInRevision& range, DUContext* parent = nullptr, bool anonymous = false);
   explicit DUContext(DUContextData&);
 
   /**
    * Destructor. Will delete all child contexts which are defined within
    * the same file as this context.
    */
-  virtual ~DUContext();
+  ~DUContext() override;
 
-  enum ContextType {
+  enum ContextType : quint8 {
     Global    /**< A context that declares functions, namespaces or classes */,
     Namespace /**< A context that declares namespace members */,
     Class     /**< A context that declares class members */,
@@ -151,7 +153,7 @@ public:
   /**
    * Find the top context.
    */
-  virtual TopDUContext* topContext() const;
+  TopDUContext* topContext() const override;
 
   /**
    * Visits all duchain objects in the whole duchain.
@@ -397,6 +399,14 @@ public:
    */
   void deleteChildContextsRecursively();
 
+  /**
+   * Resort the child contexts by their range.
+   *
+   * You must call this when you manually change the range of child contexts in a way
+   * that could break the internal range sorting.
+   */
+  void resortChildContexts();
+
   /**
    * Returns true if this declaration is accessible through the du-chain,
    * and thus cannot be edited without a du-chain write lock
@@ -435,7 +445,7 @@ public:
   QList<Declaration*> findDeclarations(const QualifiedIdentifier& identifier,
                                        const CursorInRevision& position = CursorInRevision::invalid(),
                                        const AbstractType::Ptr& dataType = AbstractType::Ptr(),
-                                       const TopDUContext* topContext = 0,
+                                       const TopDUContext* topContext = nullptr,
                                        SearchFlags flags = NoSearchFlags) const;
 
   /**
@@ -453,9 +463,17 @@ public:
    *
    * @overload
    */
+  QList<Declaration*> findDeclarations(const IndexedIdentifier& identifier,
+                                       const CursorInRevision& position = CursorInRevision::invalid(),
+                                       const TopDUContext* topContext = nullptr,
+                                       SearchFlags flags = NoSearchFlags) const;
+
+  /**
+   * Prefer the version above for speed reasons.
+   */
   QList<Declaration*> findDeclarations(const Identifier& identifier,
                                        const CursorInRevision& position = CursorInRevision::invalid(),
-                                       const TopDUContext* topContext = 0,
+                                       const TopDUContext* topContext = nullptr,
                                        SearchFlags flags = NoSearchFlags) const;
 
   /**
@@ -464,9 +482,18 @@ public:
    *
    * Does not search imported parent-contexts(like base-classes).
    */
+  QList<Declaration*> findLocalDeclarations(const IndexedIdentifier& identifier,
+                                            const CursorInRevision& position = CursorInRevision::invalid(),
+                                            const TopDUContext* topContext = nullptr,
+                                            const AbstractType::Ptr& dataType = AbstractType::Ptr(),
+                                            SearchFlags flags = NoSearchFlags) const;
+
+  /**
+   * Prefer the version above for speed reasons.
+   */
   QList<Declaration*> findLocalDeclarations(const Identifier& identifier,
                                             const CursorInRevision& position = CursorInRevision::invalid(),
-                                            const TopDUContext* topContext = 0,
+                                            const TopDUContext* topContext = nullptr,
                                             const AbstractType::Ptr& dataType = AbstractType::Ptr(),
                                             SearchFlags flags = NoSearchFlags) const;
 
@@ -490,7 +517,15 @@ public:
    * @param source A source-context that is needed to instantiate template-declarations in some cases.
    *               If it is zero, that signalizes that missing members should not be instantiated.
    */
-  virtual QVector<Declaration*> localDeclarations(const TopDUContext* source = 0) const;
+  virtual QVector<Declaration*> localDeclarations(const TopDUContext* source = nullptr) const;
+
+  /**
+   * Resort the local declarations by their range.
+   *
+   * You must call this when you manually change the range of declarations in a way
+   * that could break the internal range sorting.
+   */
+  void resortLocalDeclarations();
 
   /**
    * Searches for the most specific context for the given cursor @a position in the given @a url.
@@ -501,7 +536,7 @@ public:
    *
    * @returns the requested context if one was found, otherwise null.
    */
-  DUContext* findContext(const CursorInRevision& position, DUContext* parent = 0) const;
+  DUContext* findContext(const CursorInRevision& position, DUContext* parent = nullptr) const;
 
   /**
    * Iterates the tree to see if the provided @a context is a subcontext of this context.
@@ -539,11 +574,6 @@ public:
    */
   void cleanIfNotEncountered(const QSet<DUChainBase*>& encountered);
 
-  /**
-   * Used exclusively by Declaration, do not use this.
-   */
-  void changingIdentifier( Declaration* decl, const Identifier& from, const Identifier& to );
-
    /**
     * Uses:
     * A "Use" represents any position in a document where a Declaration is used literally.
@@ -643,9 +673,10 @@ public:
    * then the widget will not close when the cursor moves in the document, which
    * enables you to change the document contents from the widget without immediately closing the widget.
    */
-  virtual QWidget* createNavigationWidget(Declaration* decl = 0, TopDUContext* topContext = 0,
+  virtual QWidget* createNavigationWidget(Declaration* decl = nullptr, TopDUContext* topContext = nullptr,
                                           const QString& htmlPrefix = QString(),
-                                          const QString& htmlSuffix = QString()) const;
+                                          const QString& htmlSuffix = QString(),
+                                          AbstractNavigationWidget::DisplayHints hints = AbstractNavigationWidget::NoHints) const;
 
   enum {
     Identity = 2
@@ -661,10 +692,9 @@ public:
    *
    * The insertion order in the hierarchy determines the order of the represented list.
    */
-  struct KDEVPLATFORMLANGUAGE_EXPORT SearchItem : public KShared
+  struct KDEVPLATFORMLANGUAGE_EXPORT SearchItem : public QSharedData
   {
-    typedef KSharedPtr<SearchItem> Ptr;
-    ///@todo find out why this KDevVarLengthArray crashes when it's resized!
+    typedef QExplicitlySharedDataPointer<SearchItem> Ptr;
     typedef KDevVarLengthArray<Ptr, 256> PtrList;
 
     /**
@@ -683,8 +713,8 @@ public:
      */
     SearchItem(const QualifiedIdentifier& id, const PtrList& nextItems, int start = 0);
 
-    SearchItem(bool explicitlyGlobal, const Identifier& id, const PtrList& nextItems);
-    SearchItem(bool explicitlyGlobal, const Identifier& id, const Ptr& nextItem);
+    SearchItem(bool explicitlyGlobal, const IndexedIdentifier& id, const PtrList& nextItems);
+    SearchItem(bool explicitlyGlobal, const IndexedIdentifier& id, const Ptr& nextItem);
 
     bool isEmpty() const;
     bool hasNext() const;
@@ -719,7 +749,7 @@ public:
     void addNext(const Ptr& other);
 
     bool isExplicitlyGlobal;
-    Identifier identifier;
+    IndexedIdentifier identifier;
     PtrList next;
   };
 
@@ -750,18 +780,13 @@ public:
    * @return whether the search was successful. If it is false, it had to be stopped
    *         for special reasons (like some flags)
    */
-  typedef KDevVarLengthArray<Declaration*, 40> DeclarationList;
+  typedef QList<Declaration*> DeclarationList;
 
   virtual bool findDeclarationsInternal(const SearchItem::PtrList& identifiers,
                                         const CursorInRevision& position, const AbstractType::Ptr& dataType,
                                         DeclarationList& ret, const TopDUContext* source, SearchFlags flags,
                                         uint depth ) const;
 
-  /**
-   * Call this after parsing is finished. It will optimize the internal vectors to reduce memory-usage.
-   */
-  void squeeze();
-
   /**
    * Returns the qualified identifier @p id with all aliases (for example namespace imports) applied
    *
@@ -796,12 +821,14 @@ protected:
                                          const TopDUContext* source,
                                          bool searchInParents = true, int currentDepth = 0) const;
 
-  /**
-   * Logic for calculating the fully qualified scope name
-   */
-  QualifiedIdentifier scopeIdentifierInternal(DUContext* context) const;
+  void findLocalDeclarationsInternal(const Identifier& identifier,
+                                     const CursorInRevision & position,
+                                     const AbstractType::Ptr& dataType,
+                                     DeclarationList& ret,
+                                     const TopDUContext* source,
+                                     SearchFlags flags ) const;
 
-  virtual void findLocalDeclarationsInternal(const Identifier& identifier,
+  virtual void findLocalDeclarationsInternal(const IndexedIdentifier& identifier,
                                              const CursorInRevision & position,
                                              const AbstractType::Ptr& dataType,
                                              DeclarationList& ret,
@@ -831,7 +858,7 @@ protected:
    */
   virtual void applyUpwardsAliases(SearchItem::PtrList& identifiers, const TopDUContext* source) const;
 
-  DUContext(DUContextData& dd, const RangeInRevision& range, DUContext* parent = 0, bool anonymous = false);
+  DUContext(DUContextData& dd, const RangeInRevision& range, DUContext* parent = nullptr, bool anonymous = false);
 
   /**
    * Just uses the data from the given context. Doesn't copy or change anything,
@@ -859,16 +886,13 @@ protected:
   virtual bool shouldSearchInParent(SearchFlags flags) const;
 
 private:
-  void rebuildDynamicData(DUContext* parent, uint ownIndex);
+  void rebuildDynamicData(DUContext* parent, uint ownIndex) override;
 
   friend class TopDUContext;
   friend class IndexedDUContext;
   friend class LocalIndexedDUContext;
   friend class TopDUContextDynamicData;
 
-  void clearDeclarationIndices();
-  void updateDeclarationIndices();
-
   DUCHAIN_DECLARE_DATA(DUContext)
   class DUContextDynamicData* m_dynamicData;
 };
@@ -925,6 +949,6 @@ KDEVPLATFORMLANGUAGE_EXPORT QList<RangeInRevision> allUses(DUContext* context,
 
 Q_DECLARE_TYPEINFO(KDevelop::DUContext::Import, Q_MOVABLE_TYPE);
 
-#endif // KDEVPLATFORM_DUCONTEXT_H
+KDEVPLATFORMLANGUAGE_EXPORT QDebug operator<<(QDebug dbg, const KDevelop::DUContext::Import& import);
 
-// kate: space-indent on; indent-width 2; tab-width 4; replace-tabs on; auto-insert-doxygen on
+#endif // KDEVPLATFORM_DUCONTEXT_H
